"""
CITATION: The following code is a python implementation of some of the functions
taken from https://github.com/mimno/Mallet
"""

import sys, math
EULER_MASCHERONI = -0.5772156649015328606065121
PI_SQUARED_OVER_SIX = math.pi * math.pi / 6
HALF_LOG_TWO_PI = math.log(2 * math.pi) / 2
DIGAMMA_COEF_1 = 1 / 12
DIGAMMA_COEF_2 = 1 / 120
DIGAMMA_COEF_3 = 1 / 252
DIGAMMA_COEF_4 = 1 / 240
DIGAMMA_COEF_5 = 1 / 132
DIGAMMA_COEF_6 = 691 / 32760
DIGAMMA_COEF_7 = 1 / 12
DIGAMMA_COEF_8 = 3617 / 8160
DIGAMMA_COEF_9 = 43867 / 14364
DIGAMMA_COEF_10 = 174611 / 6600
DIGAMMA_LARGE = 9.5
DIGAMMA_SMALL = 0.000001

def learn_symmetric_concentration(countHistogram, observationLengths, numDimensions, currentValue):
    currentDigamma = float()
    largestNonZeroCount = 0
    nonZeroLengthIndex = [int()]*len(observationLengths)

    index = 0
    while index < len(countHistogram):
        if countHistogram[index] > 0:
            largestNonZeroCount = index
        index += 1

    denseIndex = 0
    index = 0
    while index < len(observationLengths):
        if observationLengths[index] > 0:
            nonZeroLengthIndex[denseIndex] = index
            denseIndex += 1
        index += 1

    denseIndexSize = denseIndex

    iteration = 1
    while iteration <= 200:
        currentParameter = currentValue / numDimensions
        currentDigamma = 0
        numerator = 0.0

        index = 1
        while index <= largestNonZeroCount:
            currentDigamma += 1.0 / (currentParameter + index - 1)
            numerator += countHistogram[index] * currentDigamma
            index += 1

        currentDigamma = 0
        denominator = 0.0
        previousLength = 0

        cachedDigamma = digamma(currentValue)

        denseIndex = 0
        while denseIndex < denseIndexSize:
            length = nonZeroLengthIndex[denseIndex]
            if length - previousLength > 20:
                currentDigamma = digamma(currentValue + length) - cachedDigamma
            else:
                index = previousLength
                while index < length:
                    currentDigamma += 1.0 / (currentValue + index)
                    index += 1
            denominator += currentDigamma * observationLengths[length]
            denseIndex += 1

        currentValue = currentParameter * numerator / denominator
        iteration += 1
    return currentValue


def learn_parameters(parameters, observations, observationLengths):
    return learn_params(parameters, observations, observationLengths, 1.00001, 1.0, 200)


def learn_params(parameters, observations, observationLengths, shape, scale, numIterations):
    i = int()
    k = int()
    parametersSum = 0
    k = 0
    while k < len(parameters):
        parametersSum += parameters[k]
        k += 1

    oldParametersK = float()
    currentDigamma = float()
    denominator = float()

    nonZeroLimit = int()
    nonZeroLimits = [-1]*len(observations)

    histogram = []

    i = 0
    while i < len(observations):
        histogram = observations[i]

        k = 0
        while k < len(histogram):
            if histogram[k] > 0:
                nonZeroLimits[i] = k
            k += 1
        i += 1

    iteration = 0
    while iteration < numIterations:
        denominator = 0
        currentDigamma = 0

        i = 1
        while i < len(observationLengths):
            currentDigamma += 1 / (parametersSum + i - 1)
            denominator += observationLengths[i] * currentDigamma
            i += 1

        denominator -= 1 / scale
        parametersSum = 0

        k = 0
        while k < len(parameters):
            nonZeroLimit = nonZeroLimits[k]

            oldParametersK = parameters[k]
            parameters[k] = 0
            currentDigamma = 0
            
            histogram = observations[k]
            
            i = 1
            while i <= nonZeroLimit:
                currentDigamma += 1 / (oldParametersK + i - 1)
                parameters[k] += histogram[i] * currentDigamma
                i += 1
            
            parameters[k] = oldParametersK * (parameters[k] + shape) / denominator
            parametersSum += parameters[k]
            k += 1
        iteration += 1
    if parametersSum < 0.0:
        print(parametersSum)
        print("sum not valid")
        sys.exit(1)
    return parametersSum

def digamma(z):
    psi = 0.0
    if z < DIGAMMA_SMALL:
        psi = EULER_MASCHERONI - (1 / z)

        return psi
    while z < DIGAMMA_LARGE:
        psi -= 1 / z
        z += 1
    invZ = 1 / z
    invZSquared = invZ * invZ
    psi += math.log(z) - 0.5 * invZ - invZSquared * (DIGAMMA_COEF_1 - invZSquared * (DIGAMMA_COEF_2 - invZSquared * (DIGAMMA_COEF_3 - invZSquared * (DIGAMMA_COEF_4 - invZSquared * (DIGAMMA_COEF_5 - invZSquared * (DIGAMMA_COEF_6 - invZSquared * DIGAMMA_COEF_7))))))
    return psi





from . import phrase_mining
import sys
from . import utils

arguments = sys.argv
print('Running Phrase Mining...')

file_name = arguments[1]

# represents the minimum number of occurences you want each phrase to have.
min_support=10 

# represents the threshold for merging two words into a phrase. A lower value
# alpha leads to higher recall and lower precision,
alpha=4

# length of the maximum phrase size
max_phrase_size=10

phrase_miner = phrase_mining.PhraseMining(file_name, min_support, max_phrase_size, alpha);
partitioned_docs, index_vocab = phrase_miner.mine()
frequent_phrases = phrase_miner.get_frequent_phrases(min_support)
utils.store_partitioned_docs(partitioned_docs)
utils.store_vocab(index_vocab)
utils.store_frequent_phrases(frequent_phrases)



import re
from collections import Counter
import math
import heapq
import sys
import pathlib
import os

class PhraseMining(object):
    """
    PhraseMining performs frequent pattern mining followed by agglomerative clustering
    on the input corpus and then stores the results in intermediate files.
    :param file_name:
        path to the input corpus.
    :param min_support:
        minimum support threshold which must be satisfied by each phrase during frequent
        pattern mining.
    :param max_phrase_size:
        maximum allowed phrase size.
    :param alpha:
        threshold for the significance score.
    """

    def __init__(self, documents, min_support=10, max_phrase_size=40, alpha=4):
        self.min_support = min_support
        self.max_phrase_size = max_phrase_size
        self.alpha = alpha
        self.documents = documents

    def mine(self):
        return self._run_phrase_mining(self.min_support, self.max_phrase_size, self.alpha, self.documents)

    def _frequentPatternMining(self, documents, min_support, max_phrase_size, word_freq, active_indices):
        """
        Performs frequent pattern mining to collect aggregate counts for all contiguous phrases in the 
        input document that satisfy a certain minimum support threshold.

        Parameters:
        @documents: the input corpus
        @min_support: minimum support threshold which must be satisfied by each phrase.
        @max_phrase_size: maximum allowed phrase size
        @word_freq: raw frequency of each word in the input corpus
        @active_indices: set of active indices
        """ 
        hash_counter = word_freq
        n = 2
        
        #iterate until documents is empty
        while(len(documents) > 0):
            temp_documents = []
            new_active_indices = []
            #go over each document
            for d_i,doc in enumerate(documents):
                #get set of indices of phrases of length n-1 with min support
                new_word_indices = []
                word_indices = active_indices[d_i]
                for index in word_indices:
                    words = doc.split()
                    if index+n-2 < len(words):
                        key = ""
                        for i in range(index, index+n-2+1):
                            if i == index+n-2:
                                key = key + words[i]
                            else:
                                key = key + words[i] + " "

                        #check if the phrase 'key' meets min support
                        if hash_counter[key] >= min_support:
                            new_word_indices.append(index)

                #remove the current document if there is no more phrases of length
                #n which satisfy the minimum support threshold
                if len(new_word_indices) != 0:
                    new_active_indices.append(new_word_indices)
                    temp_documents.append(doc)
                    words = doc.split()
                    for idx, i in enumerate(new_word_indices[:-1]):
                        phrase = ""
                        if (new_word_indices[idx+1] == i + 1):
                            for idx in range(i, i+n):
                                if idx == i+n-1:
                                    phrase += words[idx]
                                else:
                                    phrase += words[idx] + " "
                        hash_counter[phrase] += 1

            documents = temp_documents
            active_indices = new_active_indices
            n += 1
            if n == max_phrase_size:
                break
        
        hash_counter = Counter(x for x in hash_counter.elements() if hash_counter[x] >= min_support)
        
        return hash_counter 

    def _agglomerative_clustering(self, doc, hash_counter, alpha, total_words):
        """
        Performs agglomerative clustering to get meaningful phrases from the input document.

        Parameters:
        @doc: input corpus
        @hash_counter: map from phrases to their respective raw frequency
        @alpha: threshold for the significance score
        @total_words: total count of the words in input corpus.
        """
        sig_map = {}
        phrases = doc.split()
        while(True):
            max_sig = float("-inf")
            max_pair = -1
            for index, word in enumerate(phrases[:-1]):
                phrase = phrases[index]+" "+phrases[index+1]
                if phrase not in sig_map:
                    sig_score = self._significance_score(phrases[index], phrases[index+1], hash_counter, total_words)
                    sig_map[phrase] = sig_score
                
                if(max_sig < sig_map[phrase]):
                    max_sig = sig_map[phrase]
                    max_pair = index

            if(max_sig < alpha):
                break
                
            #merge max pair
            merged_phrase = phrases[max_pair] + " "+ phrases[max_pair+1]
            
            #fix phrases
            phrases[max_pair] = merged_phrase
            phrases.pop(max_pair+1)
        
        return phrases
            
    def _significance_score(self, phrase1, phrase2, hash_counter, total_words):
        """
        Calculates the signifance score of the phrase obtained by joining phrase1 
        and phrase2. The significance score basically measures how unlikely is the
        new phrase. The more unlikely it is, the more informative it will be.

        Parameters:
        @phrase1: first phrase
        @phrase2: second phrase
        @hash_counter: map from phrases to their respective raw frequency
        @total_words: total count of the words in input corpus.
        """
        combined_phrase = phrase1+" "+phrase2
        combined_size = len(combined_phrase.split())
        actual_occurence = hash_counter[combined_phrase]
        numerator = hash_counter[phrase1]*hash_counter[phrase2]
        
        if actual_occurence == 0:
            return float("-inf")
        
        denominator = total_words * total_words
        independent_prob = numerator/denominator
        independent_prob *= 2
        
        expected_occurence = independent_prob*total_words
        
        return (actual_occurence-expected_occurence)/math.sqrt(max(actual_occurence, expected_occurence))

    def _get_true_frequency(self, hash_counter):
        """
        Updates the raw frequency of the phrases to get their true frequencies.
        """
        true_counter = Counter(hash_counter)
        for key in hash_counter:
            val = key.split()
            if len(val) <= 1:
                continue
            substr1 = " ".join(val[0:-1])
            substr2 = " ".join(val[1:])
            true_counter[substr1] -= hash_counter[key]
            true_counter[substr2] -= hash_counter[key]

        return true_counter

    def _get_stopwords(self):
        """
        Returns a list of stopwords.
        """
        from .stopwords import stopwords
        return set(stopwords)

    def _get_word_freq(self, documents):
        """
        Calculates the frequency of each word in the input document.
        """
        total_words = 0
        word_freq = Counter()
        active_indices = []
        for doc_index, doc in enumerate(documents):
            words = doc.split()
            word_indices = []
            for word_index, word in enumerate(words):
                word_freq[word] += 1
                word_indices.append(word_index)
                total_words += 1
            active_indices.append(word_indices)

        return total_words, word_freq, active_indices

    def _get_partitioned_docs(self, document_range, doc_phrases):
        """
        Partitions the input document based on the punctuations.
        """
        partitioned_docs = []
        start = 0
        end = 0
        for idx in document_range:
            end = idx
            final_doc = []
            for i in range(start, end):
                final_doc.extend(doc_phrases[i])
            partitioned_docs.append(final_doc)
            start = end

        return partitioned_docs

    def _process_partitioned_docs(self, partitioned_docs):
        self.vocab = {}
        self.index_vocab = []
        self.partitioned_docs = []
        word_counter = 0
        for document_index, document in enumerate(partitioned_docs):
            document_of_phrases = []
            for phrase in document:
                phrases_of_words = []
                for word in phrase.split():
                    if word not in self.vocab:
                        self.vocab[word] = word_counter
                        self.index_vocab.append(word)
                        word_counter += 1
                    phrases_of_words.append(self.vocab[word])
                document_of_phrases.append(phrases_of_words)
            self.partitioned_docs.append(document_of_phrases)

    def _preprocess_input(self, original_documents, stopwords):
        """
        Performs preprocessing on the input document. Includes stopword removal.
        """
        documents = []
        document_range = []
        i = 0
        num_docs = 0
        for line in original_documents:
            line_lowercase = line.lower()
            sentences_no_punc = re.split(r"[.,;!?]",line_lowercase)
            stripped_sentences = []
            for sentence in sentences_no_punc:
                stripped_sentences.append(re.sub('[^A-Za-z0-9]+', ' ', sentence))
            sentences_no_punc = stripped_sentences
            i += len(sentences_no_punc)
            document_range.append(i)
            documents.extend(sentences_no_punc)
            num_docs += 1

        documents = [doc.strip() for doc in documents]

        # remove stop-words
        documents2 = []
        for doc in documents:
            documents2.append(' '.join([word for word in doc.split() if word not in stopwords]))

        documents = documents2[:]

        return documents, document_range, num_docs

    def _run_phrase_mining(self, min_support, max_phrase_size, alpha, documents):
        """
        Runs the phrase mining algorithm.

        Parameters:
        @min_support: minimum support threshold which must be satisfied by each phrase.
        @max_phrase_size: maximum allowed phrase size
        @alpha: threshold for the significance score
        @file_name: path to the input corpus
        """

        stopwords = self._get_stopwords()

        documents, document_range, num_docs = self._preprocess_input(documents, stopwords)

        #calculate frequency of all words
        total_words, word_freq, active_indices = self._get_word_freq(documents)

        vocab_size = len(word_freq)

        #run frequent pattern mining 
        hash_counter = self._frequentPatternMining(documents, min_support, max_phrase_size, word_freq, active_indices)

        #run agglomerative clustering
        doc_phrases = []
        for doc in documents:
            doc_phrases.append(self._agglomerative_clustering(doc, hash_counter, alpha, total_words))

        #update true count of each phrase
        self.true_counter = self._get_true_frequency(hash_counter)

        partitioned_docs = self._get_partitioned_docs(document_range, doc_phrases)
        self._process_partitioned_docs(partitioned_docs)

        return self.partitioned_docs, self.index_vocab

    def get_frequent_phrases(self, min_support):
        """
        Returns the most frequent phrases in the corpus that occur more than 
        the minimum support in descending order of frequency
        """
        frequent_phrases = []
        for key,value in self.true_counter.most_common():
            if value >= min_support and len(key.split(" "))>1:
                frequent_phrases.append((key, value))
            elif value < min_support:
                break
        return frequent_phrases




stopwords = """a
able
about
above
according
accordingly
across
actually
after
afterwards
again
against
all
allow
allows
almost
alone
along
already
also
although
always
am
among
amongst
an
and
another
any
anybody
anyhow
anyone
anything
anyway
anyways
anywhere
apart
appear
appreciate
appropriate
are
around
as
aside
ask
asking
associated
at
available
away
awfully
b
be
became
because
become
becomes
becoming
been
before
beforehand
behind
being
believe
below
beside
besides
best
better
between
beyond
both
brief
but
by
c
came
can
cannot
cant
cause
causes
certain
certainly
changes
clearly
co
com
come
comes
concerning
consequently
consider
considering
contain
containing
contains
corresponding
could
course
currently
d
definitely
described
despite
did
different
do
does
doing
done
down
downwards
during
e
each
edu
eg
eight
either
else
elsewhere
enough
entirely
especially
et
etc
even
ever
every
everybody
everyone
everything
everywhere
ex
exactly
example
except
f
far
few
fifth
first
five
followed
following
follows
for
former
formerly
forth
four
from
further
furthermore
g
get
gets
getting
given
gives
go
goes
going
gone
got
gotten
greetings
h
had
happens
hardly
has
have
having
he
hello
help
hence
her
here
hereafter
hereby
herein
hereupon
hers
herself
hi
him
himself
his
hither
hopefully
how
howbeit
however
i
ie
if
ignored
immediate
in
inasmuch
inc
indeed
indicate
indicated
indicates
inner
insofar
instead
into
inward
is
it
its
itself
j
just
k
keep
keeps
kept
know
knows
known
l
last
lately
later
latter
latterly
least
less
lest
let
like
liked
likely
little
look
looking
looks
ltd
m
mainly
many
may
maybe
me
mean
meanwhile
merely
might
more
moreover
most
mostly
much
must
my
myself
n
name
namely
nd
near
nearly
necessary
need
needs
neither
never
nevertheless
new
next
nine
no
nobody
non
none
noone
nor
normally
not
nothing
novel
now
nowhere
o
obviously
of
off
often
oh
ok
okay
old
on
once
one
ones
only
onto
or
other
others
otherwise
ought
our
ours
ourselves
out
outside
over
overall
own
p
particular
particularly
per
perhaps
placed
please
plus
possible
presumably
probably
provides
q
que
quite
qv
r
rather
rd
re
really
reasonably
regarding
regardless
regards
relatively
respectively
right
s
said
same
saw
say
saying
says
second
secondly
see
seeing
seem
seemed
seeming
seems
seen
self
selves
sensible
sent
serious
seriously
seven
several
shall
she
should
since
six
so
some
somebody
somehow
someone
something
sometime
sometimes
somewhat
somewhere
soon
sorry
specified
specify
specifying
still
sub
such
sup
sure
t
take
taken
tell
tends
th
than
thank
thanks
thanx
that
thats
the
their
theirs
them
themselves
then
thence
there
thereafter
thereby
therefore
therein
theres
thereupon
these
they
think
third
this
thorough
thoroughly
those
though
three
through
throughout
thru
thus
to
together
too
took
toward
towards
tried
tries
truly
try
trying
twice
two
u
un
under
unfortunately
unless
unlikely
until
unto
up
upon
us
use
used
useful
uses
using
usually
uucp
v
value
various
very
via
viz
vs
w
want
wants
was
way
we
welcome
well
went
were
what
whatever
when
whence
whenever
where
whereafter
whereas
whereby
wherein
whereupon
wherever
whether
which
while
whither
who
whoever
whole
whom
whose
why
will
willing
wish
with
within
without
wonder
would
would
x
y
yes
yet
you
your
yours
yourself
yourselves
z
zero
""".split()


from . import phrase_lda
import sys
from . import utils

arguments = sys.argv

num_topics = int(arguments[1])
iteration = 1100
optimization_burnin = 100
alpha = 4
optimization_iterations = 50
beta = 0.01

print('Running PhraseLDA...')

partitioned_docs = utils.load_partitioned_docs()
vocab_file = utils.load_vocab()

plda = phrase_lda.PhraseLDA( partitioned_docs, vocab_file, num_topics , alpha, beta, iteration, optimization_iterations, optimization_burnin);

document_phrase_topics, most_frequent_topics = plda.run()
utils.store_phrase_topics(document_phrase_topics)
utils.store_most_frequent_topics(most_frequent_topics)



from . import dirichlet
import random
from . import utils
import math
from collections import Counter


class PhraseLDA(object):

    """ 
    Topic Modeling using lda with collapsed gibbs sampling on "bag-of-phrases".
    :param partitioned_docs:
        List of documents, where each document is partitioned into a list of 
        phrases. 
    :param index_vocab:
        Mapping of integer index to string word.
    :param num_topics:
        Number of requested topics that need to be extracted from the inputted
        documents.
    :param alpha:
        Scalar hyperparameter representing the symmetric Dirichlet prior over 
        document-topic (lambda) distributions. Initially, each topic will have 
        the same prior weight for each document.
    :param beta:
        Scalar hyperparameter representing the symmetric Dirichlet prior over 
        topic-word (phi) distributions. Initially, each word will have the same 
        prior weight for each topic.
    :param iterations:
        The total number of Gibbs sampling iterations over the entire corpus.
    :param optimization_iterations:
        Sets hyperparameter optimization after each ``optimization_iterations``
        of the Gibbs sampling. Set to ``None```` for no optimization.
    :param optimization_burnin:
        Number of gibbs sampling iterations before hyperparameter optimization
        starts.
    """

    def __init__(self, partitioned_docs, index_vocab, 
                 num_topics=5, alpha=4, beta=0.01, iterations=1000,
                 optimization_iterations=100, optimization_burnin=50):
        # initialize corpus
        self.documents = partitioned_docs
        self.num_documents = len(partitioned_docs)
        self.index_vocab = index_vocab
        self.num_words = len(index_vocab)
        self.num_topics = num_topics

        # initialize hyperparameters
        self.alpha = [alpha] * self.num_topics
        self.alpha_sum = alpha * num_topics
        self.beta = beta
        self.beta_sum = self.beta * self.num_words
        
        # gibbs sampling parameters
        self.iterations = iterations
        self.optimization_iterations = optimization_iterations
        self.optimization_burnin = optimization_burnin

    def _initialize(self):
        self._init_documents()

        # Array stores per topic counts
        self.n_t = [0] * self.num_topics

        # 2d array that stores document/topic counts by phrase, and word respectively
        self.n_d_t_phrases = [[0] * self.num_topics for __ in range(self.num_documents)]
        self.n_d_t_words = [[0] * self.num_topics for __ in range(self.num_documents)]

        # 2d array that stores topic/word counts
        self.n_t_w = [[0] * self.num_words for __ in range(self.num_topics)]

        self._init_documents_topics()

        self._init_histogram()

    def _init_documents(self):
        self.documents_words = []

        self.max_documents_phrases_count = 0
        self.max_documents_words_count = 0

        for document in self.documents:
            document_words = []

            document_words_count = 0

            for phrase in document:
                for word in phrase:
                    document_words.append(word)

                    document_words_count += 1

            self.documents_words.append(document_words)

            self.max_documents_phrases_count = max(self.max_documents_phrases_count, len(document))
            self.max_documents_words_count = max(self.max_documents_words_count, document_words_count)

    def _init_documents_topics(self):
        # we assign a random topic to each phrase in the document
        self.documents_phrases_topic = []

        for document_index, document in enumerate(self.documents):
            document_phrases_topic = []
            for phrase_index, phrase in enumerate(document):
                document_phrase_topic = random.randint(0,self.num_topics-1)
                document_phrases_topic.append(document_phrase_topic)
                
                # Increase counts
                self.n_t[document_phrase_topic] += len(phrase)
                self.n_d_t_phrases[document_index][document_phrase_topic] += 1
                self.n_d_t_words[document_index][document_phrase_topic] += len(phrase)
                for word_index in phrase:
                    self.n_t_w[document_phrase_topic][word_index] += 1
            self.documents_phrases_topic.append(document_phrases_topic)


    def _init_histogram(self):
        self.document_length_histogram = [0] * (self.max_documents_words_count + 1)
        for document in self.documents_words:
            self.document_length_histogram[len(document)] += 1
        self._init_topic_document_histogram()

    def _init_topic_document_histogram(self):
        self.topic_document_histogram = [[int()] * (self.max_documents_words_count + 1)
                                         for __ in range(self.num_topics)]

    def _sample_topic(self, sampling_probabilities):
        threshold = random.uniform(0.0,1.0) * sum(sampling_probabilities)
        cumulative_sum = 0
        for topic in range(self.num_topics):
            cumulative_sum += sampling_probabilities[topic]
            if cumulative_sum > threshold:
                break
        return topic

    def _calculate_topic_probabilities(self, document_index, phrase_index):
        topic_probabilities = []
        for topic_index in range(self.num_topics):
            left = self.alpha[topic_index] + self.n_d_t_phrases[document_index][topic_index]
            right = 1.0
            for word_index in self.documents[document_index][phrase_index]:
                right *= (self.beta + self.n_t_w[topic_index][word_index]) / (self.beta_sum + (self.n_t[topic_index]))
            topic_probability = left * right
            topic_probabilities.append(topic_probability)
        return topic_probabilities

    def _should_optimize(self, iterations):
        if self.optimization_iterations is None:
            return false
        iterations_condition = ((iterations+1) % self.optimization_iterations) == 0
        burnin_condition = ((iterations+1) > self.optimization_burnin)
        return iterations_condition and burnin_condition


    def run(self):
        self._initialize()        
        for iteration in range(self.iterations):
            if iteration % 100 == 0:
                print("iteration", iteration)

            for document_index, document in enumerate(self.documents):
                for phrase_index, phrase in enumerate(document):
                    document_phrase_topic = self.documents_phrases_topic[document_index][phrase_index]

                    # reduce counts for sampling
                    self.n_t[document_phrase_topic] -= len(phrase)
                    self.n_d_t_phrases[document_index][document_phrase_topic] -= 1
                    self.n_d_t_words[document_index][document_phrase_topic] -= len(phrase)
                    for word_index in phrase:
                        self.n_t_w[document_phrase_topic][word_index] -= 1

                    sampling_probabilities = self._calculate_topic_probabilities(document_index, phrase_index)
                    document_phrase_topic = self._sample_topic(sampling_probabilities)

                    self.documents_phrases_topic[document_index][phrase_index] = document_phrase_topic
                    
                    self.n_t[document_phrase_topic] += len(phrase)
                    self.n_d_t_phrases[document_index][document_phrase_topic] += 1
                    self.n_d_t_words[document_index][document_phrase_topic] += len(phrase)
                    for word_index in phrase:
                        self.n_t_w[document_phrase_topic][word_index] += 1

            if self._should_optimize(iteration):
                self._optimize_hyperparameters()
        
        topics = self._getTopics()
        return self.documents_phrases_topic, self._getMostFrequentPhrasalTopics(topics)

    def _optimize_hyperparameters(self):
        self._init_topic_document_histogram()
        for topic_index in range(self.num_topics):
            for document_index in range(len(self.documents)):
                self.topic_document_histogram[topic_index][self.n_d_t_words[document_index][topic_index]] += 1

        self.alpha_sum = dirichlet.learn_parameters(
            self.alpha, self.topic_document_histogram, self.document_length_histogram)
        max_topic_size = 0

        for topic_index in range (self.num_topics):
            if self.n_t[topic_index] > max_topic_size:
                max_topic_size = self.n_t[topic_index]

        topic_size_histogram = [0] * (max_topic_size + 1)
        count_histogram = [0] * (max_topic_size + 1)

        topic_index = 0
        for topic_index in range(self.num_topics):
            topic_size_histogram[self.n_t[topic_index]] += 1
            for word_index in range(self.num_words):
                count_histogram[
                    self.n_t_w[topic_index][word_index]] += 1

        self.beta_sum = dirichlet.learn_symmetric_concentration(
            count_histogram, topic_size_histogram, self.num_words, self.beta_sum)
        self.beta = self.beta_sum / self.num_words

    def store_phrase_topics(self, path):
        f = open(path, 'w')
        for document in self.documents_phrases_topic:
            f.write(",".join(str(phrase) for phrase in document))
            f.write("\n")

    def _getTopics(self):
        """
        Returns the set of phrases modelling each document.
        """
        topics = []
        for i in range(self.num_topics):
            topics.append(Counter())
        for document_index, document in enumerate(self.documents_phrases_topic):
            for phrase_index, phrase_topic in enumerate(document):
                phrase = " ".join(str(word) for word in self.documents[document_index][phrase_index])
                topics[phrase_topic][phrase] += 1
        return topics

    def _getMostFrequentPhrasalTopics(self, topics):
        output = []
        topic_index = 0
        for topic in topics:
            output_for_topic = []
            print("topic", topic_index)
            for phrase, count in topic.most_common():
                if len(phrase.split(" ")) > 1:
                    val = utils._get_string_phrase(phrase, self.index_vocab), count
                    output_for_topic.append(val)
                    print(val)
            output.append(output_for_topic)
            topic_index += 1
        return output



def store_partitioned_docs(partitioned_docs, path="intermediate_output/partitioneddocs.txt"):
    f = open(path, 'w')
    for document in partitioned_docs:
        f.write(", ".join(" ".join(str(word) for word in phrase) for phrase in document))
        f.write("\n")

def load_partitioned_docs(path="intermediate_output/partitioneddocs.txt"):
    f = open(path, 'r')
    partitioned_docs = []
    document_index = 0
    for line in f:
        line = line.strip()
        if len(line) < 1:
            continue
        phrases = line.split(", ")
        partitioned_doc = []
        for phrase in phrases:
            phrase_of_words = list(map(int,phrase.split(" ")))
            partitioned_doc.append(phrase_of_words)
        partitioned_docs.append(partitioned_doc)
    return partitioned_docs

def store_vocab(index_vocab, path="intermediate_output/vocab.txt"):
    """
    Stores vocabulary into a file.
    """
    f = open('intermediate_output/vocab.txt', 'w')
    for word in index_vocab:
        f.write(word+"\n")
    f.close()

def load_vocab(path="intermediate_output/vocab.txt"):
    """
    Loads vocabulary from a file.
    """
    f = open(path, 'r')
    index_vocab = []
    index = 0
    for line in f:
        index_vocab.append(line.replace("\n", ""))
    return index_vocab

def store_frequent_phrases(frequent_phrases, path='output/frequent_phrases.txt'):
    f = open(path, 'w')
    for phrase, val in enumerate(frequent_phrases):
        f.write(str.format("{0} {1}\n",phrase, val))
    f.close()

def store_phrase_topics(document_phrase_topics, path="intermediate_output/phrase_topics.txt"):
    """
    Stores topic for each phrase in the document.
    """
    f = open(path, 'w')
    for document in document_phrase_topics:
        f.write(",".join(str(phrase) for phrase in document))
        f.write("\n")

def store_most_frequent_topics(most_frequent_topics, prefix_path="output/topic"):
    for topic_index, topic in enumerate(most_frequent_topics):
        file_name = str.format("{0}{1}.txt", prefix_path, topic_index)
        f = open(file_name, 'w')
        for phrase, val in topic:
            f.write(str.format("{0} {1}\n",phrase, val))
        f.close()

def _get_string_phrase(phrase, index_vocab):
    """
    Returns the string representation of the phrase.
    """
    res = ""
    for vocab_id in phrase.split():
        if res == "":
            res += index_vocab[int(vocab_id)]
        else:
            res += " " + index_vocab[int(vocab_id)]
    return res



from setuptools import setup
import setuptools
setup(
    name='topmine',
    version='0.1.1',
    description='topic mining',
    url='https://github.com/WangHexie/topmine',
    author='UNKNOWN',
    author_email='test@test.com',
    license='BSD 2-clause',
    # package_dir={"": "topmine"},
    packages=["topmine"],
    install_requires=[
                      'numpy',
                      ],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',

    ],
)
